function [theW,theD,costs,mincost,mineps,testerrs] = run_code_example()
%  run_code_example(gammas,trainx,trainy,testx,testy,task_indexes,task_indexes_test,cv_size,Dini,iterations,...
%    method_str, epsilon_init, fname)

% Example script of running the train_alternating_epsilon() code
% Uses regularizer trace( W' D^{-1} W ) and square loss (see file
% kernel_regression.m)

% INPUTS:
% gammas: a vector of the gammas to select from using cross-validation
% trainx: features x examples matrix (dxN)
% task_indexes: starting indexes for each task in data 
% cv_size: number of cross validations performed 
% Dini: the initial matrix D 
% epsilon_init : perturbation epsilon (use 0 for no perturbation)
% method_str : see below
% See train_alternating.m for the rest

% OUTPUTS:
% bestcv: the best cross-validation performance among the gammas tried
% bestgamma: the best gamma found using cross-validation
% theW: a matrix for which each column is a w_t
% theD: matrix D estimated

clear; clc; close all;

%
N = 100; % Num tasks.
n = 100; % Num features.


% Syenthetic train data 
for ii = 1 : N 
    j = randi(500); % Random training examples.
    data(ii).X = randn(j, n); % X for task ii.
    data(ii).y = randn(j, 1); % y for task ii.
end



% Synthetic test set
for ii = 1 : N 
    j = randi(500); % Random test examples
    data_test(ii).X = randn(j, n); % X for task ii.
    data_test(ii).y = randn(j, 1); % y for task ii.
end

problem.data = data;

% When you have data_test that follows the
% same structure of data
problem.data_test = data_test;


gammas = 1;
T = 10;
d = 20;
m = 5;
N = m*T;
trainx = rand(d,N);
trainy = rand(N,1);
m_test = 6;
N_test = m_test*T;
testx = rand(d,N_test);
testy = rand(N_test,1);
task_indexes = nan(T,1);
task_indexes_test = nan(T,1);
for t=1:(T)
    task_indexes(t) = m*(t-1)+1;
    task_indexes_test(t) = m_test*(t-1)+1;
end
Dtemp = rand(d,d);
Dini = Dtemp*Dtemp';
Dini = Dini + 1000*eps*eye(size(Dini));
Dini = Dini./trace(Dini);
iterations = 1000;
method_str = 'feat';
epsilon_init = 1e-8;
fname = 'temp';

feat = 1; independent = 2; diagonal = 3;

if (strcmp(method_str,'feat'))
    method = feat;
elseif (strcmp(method_str,'ind'))
    method = independent;
elseif (strcmp(method_str,'diag'))
    method = diagonal;
else
    error('Wrong method');
end
% (see file train_alternating.m)

% Define method for computing f(D)
function v = vec_inv(d)
    v = zeros(length(d),1);
    ind = find(d > eps);
    v(ind) = 1 ./ d(ind);
end
% bestcv = [];
% bestgamma = [];
% cverrs = [];

[theW,theD,costs,mincost,mineps] = train_alternating_epsilon(trainx, trainy, task_indexes, gammas, Dini, iterations, ...
    method, 'kernel_regression', @vec_inv, @(b)(b/sum(b)), epsilon_init);

testerrs = mean(test_error_unbalanced(theW,testx,testy,task_indexes_test));

% save(sprintf('results_%s_%s_lin.mat',fname,method_str),'gammas','Dini','method_str',...
%     'testerrs','theW','theD','costs','mineps');

keyboard
end