function [Omega, W, infos] = train_alternating_epsilon_Argyriou(problem,Omegainit,params)
    % We solve the optimization problem
    % <invOmega, WW'> + lambda*sum_i||X_i w_i - y_i ||^2 + mu*<invOmega, I>
    % with unit trace constraint on Omega and Omega is symmetric positive
    % definite.
    % Multitask version.
    
    % Problem definition
    % problem.data % is a struct array.
    N = length(problem.data); % Number of tasks.
    n = size(problem.data(1).X, 2); % Number of features.
    
    for t = 1 : N % Number of tasks
        problem.data(t).XtX = problem.data(t).X'*problem.data(t).X;
        problem.data(t).Xty = problem.data(t).X'*problem.data(t).y;
    end
    
    Xtymat = zeros(n,N);
    for t = 1: N % Number of tasks
        Xtymat(:,t) = problem.data(t).Xty;
    end
    
    if ~isempty(Omegainit)
        D = diag(1+rand(n, 1));
        [Q, R] = qr(randn(n)); %#ok<NASGU>
        Omegainit = Q*D*Q';
        Omegainit = Omegainit/trace(Omegainit); % Trace is 1.
    end
    
    % Options;
    if ~isfield(params, 'mu'); params.mu = 1e-8; end
    if ~isfield(params, 'lambda'); params.lambda = 1; end
    if ~isfield(params, 'computestats'); params.computestats = false; end;
    if ~isfield(params, 'maxiter'); params.maxiter = 100; end;
    if ~isfield(params, 'maxinner'); params.maxinner = 40; end;
    %     if ~isfield(params, 'method'); params.method = @trustregions; end;
    %     if ~isfield(params, 'geometry'); params.geometry = 'flat'; end;
    mu = params.mu;
    lambda = params.lambda;
    
    %     [theW,theD,costs,mincost,mineps] = train_alternating_epsilon(trainx, trainy, task_indexes, gammas, Dini, iterations, ...
    %         method, 'kernel_regression', @vec_inv, @(b)(b/sum(b)), epsilon_init);
    %     [W,D,costfunc,mincost,mineps] = train_alternating_epsilon(trainx,trainy,task_indexes,gamma,Dini,iterations,...
    %         method,kernel_method,f_method,Dmin_method,epsilon_init)
    Dmin_e_method = @(b) (feval(@(c)(c/sum(c)), sqrt(b.^2+mu)));
    infos.cost = nan(params.maxiter,1);
    infos.objective = nan(params.maxiter,1);
    infos.train = nan(params.maxiter,1); % Only the train loss.
    infos.test = nan(params.maxiter,N); % iterations and number of tasks.
    infos.disttoUopt = nan(params.maxiter,1); % Distance to target subspace if given.
    infos.time = nan(params.maxiter,1);
    train_alternating();
    
    function train_alternating()
        Omega = Omegainit;
        %         costfunc = [];
        %         % Compute f(D)^{-1/2} for the next step
        %         [U,S,~] = svd(Omega);               % svd seems more robust than eig
        %         fS = vec_inv(diag(S));
        %         temp = sqrt(fS);
        %         tempi = find(temp > eps);
        %         temp(tempi) = 1./temp(tempi);
        %         fD_isqrt = U * diag(temp) * U';
        for iter = 1:params.maxiter
            tic;
            % Use variable transform to solve the regularization problem for
            % fixed D
            %             new_trainx = fD_isqrt * trainx;
            %             [W,costf,err,reg] = train_kernel(new_trainx,trainy,task_indexes,gamma,kernel_method);
            %             W = fD_isqrt * W;
            if iter==1
                invOmega = pinv(Omega);
            end
            W = computeW(invOmega);
            %             costfunc = [costfunc; iter, costf, err, reg];
            [U,S,~] = svd(W);

            tignore = tic;
            loss = 0;
            for t1 = 1 : N
                temp = problem.data(t1).X*W(:,t1) - problem.data(t1).y;
                loss = loss + temp'*temp;
            end
            infos.cost(iter) = mu*trace(invOmega) + trace(W*W'*invOmega);% + lambda*loss;
            infos.objective(iter) = infos.cost(iter) + lambda*loss;
            infos.train(iter) = loss;
            fprintf('[%0.4d] Cost: %+e\n',iter,infos.cost(iter));
            for i = 1 : N % tasks
                infos.test(iter, i) = calculateErr(problem.data_test(i).X*W(:,i), problem.data_test(i).y, 'nmse');
            end
            if isfield(problem,'Uopt')
                infos.disttoUopt(iter,1) = subspace(problem.Uopt, U);
                subspace(problem.Uopt, U)
            end
            time_ignore = toc(tignore);
            
            
            
            
            %             keyboard;
            %             infos.cost(iter) = mu*sum(OnebySmin) + (diagS.^2)'*OnebySmin + lambda*loss;
            % Update D
            if (n > N)
                S = [S, zeros(n,n-N)];
            end
            diagS = diag(S);
            Smin = feval(Dmin_e_method, diagS);
            OnebySmin = 1./Smin;
            invOmega = U * diag(OnebySmin) * U';
            infos.time(iter) = toc - time_ignore;
            
            
            %             % Compute f(D)^{-1/2} for the next step
            %             fS = vec_inv(Smin);
            %             temp = sqrt(fS);
            %             tempi = find(temp > eps);
            %             temp(tempi) = 1./temp(tempi);
            %             fD_isqrt = U * diag(temp) * U';
        end
        
        
        tic;
        Omega = U * diag(Smin) * U';
        infos.time(iter) = infos.time(iter) + toc;
    end
    
    % Compute W given Omega
    function W = computeW(invOmega)
        W = zeros(n, N); % Tasks as columns.
        for i = 1 : N % Number of tasks
            XitXi = problem.data(i).XtX;
            Xityi = problem.data(i).Xty;
            W(:,i) = lambda*((invOmega + lambda*(XitXi))\(Xityi));
        end
    end
    %     % Define method for computing f(D)
    %     function v = vec_inv(d)
    %         v = zeros(length(d),1);
    %         ind = find(d > eps);
    %         v(ind) = 1 ./ d(ind);
    %     end
end
