function [U_ii_new, U_jj_new] = Grassmann_update(ii, jj, U_cells, problem, options)
    % Take a gradient update.
    % U_ii ---> U_ii_new
    % U_jj ---> U_jj_new
    
    % Part of the codes for Riemannian gossip algorithms for
    % decentralized matrix completion.
    % Original authors: Bamdev Mishra <bamdevm@gmail.com> and
    %                   Hiroyuki Kasai <kasai@is.uec.ac.jp>.
    
    % Problem information
    m = problem.m;
    r = problem.r;
    N = problem.N;
    
    
    % Options
    rho = options.rho;
    stepsize = options.stepsize;
    
    % Define the Grassmann manifold
    problem.M = grassmannfactory(m, r);
    
    
    % Compute the factors at ii and jj nodes.
    U_ii = U_cells{ii};
    U_jj = U_cells{jj};
    
    
    % Balancing the probabilities.
    alpha_ii = 0.5;
    alpha_jj = 0.5;
    
    if ii == 1
        alpha_ii = 1;
    end
    
    if ii == N - 1
        alpha_jj = 1;
    end
    
    
    % Gradients at the nodes.
    % It contains a weighted sum of objective function plus sharing.
    grad_ii = computegradcost(U_ii, ii, problem, options);
    grad_ii = alpha_ii*grad_ii ...
        - rho*problem.M.log(U_ii, U_jj);
    
    %     % Debug
    %     norm(U_ii'*grad_ii, 'fro')
    
    grad_jj = computegradcost(U_jj, jj, problem, options);
    grad_jj = alpha_jj*grad_jj ...
        - rho*problem.M.log(U_jj, U_ii);
    
    
    % Take a gradient step at both ii and jj nodes.
    U_ii_new = problem.M.exp(U_ii, -grad_ii, stepsize);
    U_jj_new = problem.M.exp(U_jj, -grad_jj, stepsize);
    
    
end

