clear; clc; close all;



%% Real data
trainTestRatio = 0.8; %% between 0 and 1, larger implies larger training data
seed = 42; % setting randomization seed for consistent output
dataset = 'school'; % values permitted: 'sarcos', 'school', 'parkinson'. 

% Details of datasets present in the file generateTrainTestSplit.m
[data1,data1_test] = generateTrainTestSplit(dataset,trainTestRatio);

% Train data
problem.data = data1;

% Test data
problem.data_test = data1_test;


%% Proposed

% Mandatory information.
problem.r = 4; % Rank
problem.N = 10; % Number of agents
problem.m = size(problem.data(1).X, 2); % Number of features


% Options not mandatory. If using default options, replace options = [].
options.stepsize = 1e-7; % Initial stepsize.
options.rho = 1e5; % The weight factor.
options.lambda = 1e-1; % The ridge regression parameter for computing W.
options.stepsize_decay = 1e3; % The stepsize decay rate.
options.iters = 400; % Number of iterations.
options.verbosity = 1; % Verbosity.



% Call parallel algorithm
[U_cells, infos, options] = parallel_decentralized_MTL_feat([], problem, options);


% NMSE assuming that the agents are in consensus.
T = length(problem.data); % Total number of tasks. 
nmse = 0;
for i = 1:problem.N
    nmse = nmse + length(infos{i}.tasks)*infos{i}.cost_test(end);
end
nmse = nmse/T;

% NMSE after refinement with Karcher mean computation.
Uopt = karchermean_Grassmann(U_cells); % This gives an "average" point.
nmse_refined = 0;
for t = 1:T
    Xi = problem.data(t).X;
    yi = problem.data(t).y;
    XitXi = Xi'*Xi;
    Xityi = Xi'*yi;
    w = (options.lambda*eye(problem.r) + (Uopt'*XitXi)*Uopt)\(Uopt'*Xityi);
    nmse_refined = nmse_refined + calculateErr(problem.data_test(t).X*(Uopt*w), problem.data_test(t).y, 'nmse');
end
nmse_refined = nmse_refined/T;





%% Plots

pp = 10;

% Consensus
fs = 20;
figure;
semilogy(infos{1}.commcost(1:pp:end),'-+','Color','m','LineWidth',2, 'MarkerSize',13);
hold on;
semilogy(infos{2}.commcost(1:pp:end),'-*','Color','b','LineWidth',2, 'MarkerSize',13);
semilogy(infos{3}.commcost(1:pp:end),'-d','Color','k','LineWidth',2, 'MarkerSize',13);
semilogy(infos{4}.commcost(1:pp:end),'-O','Color','r','LineWidth',2, 'MarkerSize',13);
hold off;
ax1 = gca;
set(ax1,'FontSize',fs);
xlabel(ax1,['Every ',num2str(pp),'th update'],'FontSize',fs);
ylabel(ax1,'Consensus','FontSize',fs);
legend('Distance 1 - 2', 'Distance 2 - 3','Distance 3 - 4','Distance 4 - 5');
legend 'boxoff';
title('Parallel algorithm');
box off;



% Train
fs = 20;
figure;
plot(infos{1}.cost(1:pp:end),'-+','Color','m','LineWidth',2, 'MarkerSize',13);
hold on;
plot(infos{2}.cost(1:pp:end),'-*','Color','b','LineWidth',2, 'MarkerSize',13);
plot(infos{3}.cost(1:pp:end),'-d','Color','k','LineWidth',2, 'MarkerSize',13);
plot(infos{4}.cost(1:pp:end),'-O','Color','r','LineWidth',2, 'MarkerSize',13);
hold off;
ax1 = gca;
set(ax1,'FontSize',fs);
xlabel(ax1,['Every ',num2str(pp),'th update'],'FontSize',fs);
ylabel(ax1,'Train cost','FontSize',fs);
legend('Agent 1', 'Agent 2', 'Agent 3', 'Agent 4');
legend 'boxoff';
title('Parallel algorithm');
box off;



% Test NMSE
fs = 20;
figure;
plot(infos{1}.cost_test(1:pp:end),'-+','Color','m','LineWidth',2, 'MarkerSize',13);
hold on;
plot(infos{2}.cost_test(1:pp:end),'-*','Color','b','LineWidth',2, 'MarkerSize',13);
plot(infos{3}.cost_test(1:pp:end),'-d','Color','k','LineWidth',2, 'MarkerSize',13);
plot(infos{4}.cost_test(1:pp:end),'-O','Color','r','LineWidth',2, 'MarkerSize',13);
hold off;
ax1 = gca;
set(ax1,'FontSize',fs);
xlabel(ax1,['Every ',num2str(pp),'th update'],'FontSize',fs);
ylabel(ax1,'Mean NMSE on the test set','FontSize',fs);
legend('Agent 1', 'Agent 2', 'Agent 3', 'Agent 4');
legend 'boxoff';
title('Parallel algorithm');
box off;





