clear; clc; close all;



%% Real data
trainTestRatio = 0.8; %% between 0 and 1, larger implies larger training data
seed = 42; % setting randomization seed for consistent output
dataset = 'school'; % values permitted: 'sarcos', 'school', 'parkinson'. 

% Details of datasets present in the file generateTrainTestSplit.m
[data1,data1_test] = generateTrainTestSplit(dataset,trainTestRatio);

% Data
problem.data = data1;

% When you have data_test that follows the
% same structure of data
problem.data_test = data1_test;

% Initialization
m = size(problem.data(1).X, 2); % Number of features
T = length(problem.data); % Total number of tasks.




%% Proposed
% problem.r = 9; % 3 works good for School, 4 for parkinson.
% problem.N = 6;
% problem.m = m;
% N = problem.N;
% clear options;
% 
% % Online
% options.iters = (N-1)*200;
% options.stepsize = 1e-6; % 1e-6 for school,parkinson.
% options.rho = 1e6; % 1e6 for school, parkinson.
% options.lambda = 1; % 1 for school, 1e-3 for parkinson.
% options.stepsize_decay = 1e3; 1e-3/options.stepsize; 1e3;  1e-1/options.stepsize; %1e3 for school, 1e4 for parkinson.
% [U_cells, infos] = online_decentralized_MTL_feat([], problem, options);
% 
% 
% % % Parallel
% % options.iters = 400; %(N-1)*400;
% % options.stepsize = 1e-7;
% % [U_cells, infos] = parallel_decentralized_MTL_feat([], problem, options);
% 
% 
% % NMSE assuming that the agents are in consensus.
% nmse = 0;
% for i = 1:problem.N
%     nmse = nmse + length(infos{i}.tasks)*infos{i}.cost_test(end);
% end
% nmse = nmse/T
% 
% % NMSE after refinement with Karcher mean computation.
% Uopt = karchermean_Grassmann(U_cells);
% nmse1 = 0;
% for t = 1:T
%     Xi = problem.data(t).X;
%     yi = problem.data(t).y;
%     XitXi = Xi'*Xi;
%     Xityi = Xi'*yi;
%     w = (options.lambda*eye(problem.r) + (Uopt'*XitXi)*Uopt)\(Uopt'*Xityi);
%     nmse1 = nmse1 + calculateErr(problem.data_test(t).X*(Uopt*w), problem.data_test(t).y, 'nmse');
% end
% nmse1 = nmse1/T




%% Alt minimization Algorithm of Argyriou et al (2008)
% For Parkinson, mu = 1, lambda = 1e3.
% For School, mu = 1, lambda = 1.
% For Sarcos, mu = 1, lambda = 1.


clear options;
mu = 1e-8;
lambda = 0.5*1;
maxiter = 2*200;


options.mu = mu;
options.lambda = lambda;
options.maxiter= maxiter;
options.computestats = true; % If we want to compute the test rmsse, make it true.

[Omega_alt, W_alt, infos_alt] = train_alternating_epsilon_Argyriou(problem,eye(m)/m,options);
test_error_alt = infos_alt.test;
nmse_alt = mean(infos_alt.test(end, :))

%% Plots
pp = 10;

% % Consensus
% fs = 20;
% figure;
% semilogy(infos{1}.commcost(1:pp:end),'-+','Color','m','LineWidth',2, 'MarkerSize',13);
% hold on;
% semilogy(infos{2}.commcost(1:pp:end),'-*','Color','b','LineWidth',2, 'MarkerSize',13);
% semilogy(infos{3}.commcost(1:pp:end),'-d','Color','k','LineWidth',2, 'MarkerSize',13);
% semilogy(infos{4}.commcost(1:pp:end),'-O','Color','r','LineWidth',2, 'MarkerSize',13);
% semilogy(infos{5}.commcost(1:pp:end),'-s','Color','c','LineWidth',2, 'MarkerSize',13);
% hold off;
% ax1 = gca;
% set(ax1,'FontSize',fs);
% xlabel(ax1,['Every ',num2str(pp),'th update'],'FontSize',fs);
% ylabel(ax1,'Consensus','FontSize',fs);
% legend('Consensus 1 - 2', 'Consensus 2 - 3','Consensus 3 - 4','Consensus 4 - 5','Consensus 5 - 6');
% legend 'boxoff';
% box off;



% % Train
% fs = 20;
% figure;
% plot(infos{1}.cost(1:pp:end),'-+','Color','m','LineWidth',2, 'MarkerSize',13);
% hold on;
% plot(infos{2}.cost(1:pp:end),'-*','Color','b','LineWidth',2, 'MarkerSize',13);
% plot(infos{3}.cost(1:pp:end),'-d','Color','k','LineWidth',2, 'MarkerSize',13);
% plot(infos{4}.cost(1:pp:end),'-O','Color','r','LineWidth',2, 'MarkerSize',13);
% hold off;
% ax1 = gca;
% set(ax1,'FontSize',fs);
% xlabel(ax1,['Every ',num2str(pp),'th update'],'FontSize',fs);
% ylabel(ax1,'Train cost','FontSize',fs);
% legend('Agent 1', 'Agent 2', 'Agent 3', 'Agent 4');
% legend 'boxoff';
% box off;



% Test NMSE
avg_test = mean(infos_alt.test, 2);
% avg_test = nmse_alt*ones(size(infos_alt.test,1),1);

fs = 20;
figure;
plot(infos{1}.cost_test(1:pp:end),'-+','Color','m','LineWidth',2, 'MarkerSize',8);
hold on;
plot(infos{2}.cost_test(1:pp:end),'-*','Color','b','LineWidth',2, 'MarkerSize',8);
plot(infos{3}.cost_test(1:pp:end),'-d','Color','k','LineWidth',2, 'MarkerSize',8);
plot(infos{4}.cost_test(1:pp:end),'-O','Color','r','LineWidth',2, 'MarkerSize',8);
semilogy(infos{5}.cost_test(1:pp:end),'-s','Color','c','LineWidth',2, 'MarkerSize',8);
semilogy(infos{6}.cost_test(1:pp:end),'-v','Color','g','LineWidth',2, 'MarkerSize',8);
plot(avg_test(1:pp:end),'--','Color','k','LineWidth',2, 'MarkerSize',8);
hold off;
ax1 = gca;
set(ax1,'FontSize',fs);
xlabel(ax1,['Every ',num2str(pp),'th update'],'FontSize',fs);
ylabel(ax1,'Mean NMSE on the test set','FontSize',fs);
legend('Agent 1', 'Agent 2', 'Agent 3', 'Agent 4','Agent 5','Agent 6','Alt min');
legend 'boxoff';
box off;





