clear; clc; close all;



%% Real data
trainTestRatio = 0.8; %% between 0 and 1, larger implies larger training data
seed = 42; % setting randomization seed for consistent output
dataset = 'school'; % values permitted: 'sarcos', 'school', 'parkinson'. 

% Details of datasets present in the file generateTrainTestSplit.m
[data1,data1_test] = generateTrainTestSplit(dataset,trainTestRatio);

% Train data
problem.data = data1;

% Test data
problem.data_test = data1_test;

% Mandatory information.
problem.N = 11; % Number of agents
problem.m = size(problem.data(1).X, 2); % Number of features
problem.r = 5; % Rank

%% Proposed

% % Options not mandatory. If using default options, replace options = [];
options.stepsize = 1e-6; % Initial stepsize.
options.rho = 1e5; % The weight factor.
options.lambda = 1e-1; % The ridge regression parameter for computing W.
options.stepsize_decay = 1e3; % The stepsize decay rate.
options.iters = (problem.N-1)*200; % Number of iterations.
options.verbosity = 1; % Verbosity.



% Call Online algorithm
[U_cells_online, infos_online, options_online] = online_decentralized_MTL_feat([], problem, options);


% NMSE assuming that the agents are in consensus.
T = length(problem.data); % Total number of tasks.
nmse = 0;
for i = 1:problem.N
    nmse = nmse + length(infos_online{i}.tasks)*infos_online{i}.cost_test(end);
end
nmse_online = nmse/T;

% NMSE after refinement with Karcher mean computation.
Uopt = karchermean_Grassmann(U_cells_online); % This gives an "average" point.
nmse_refined = 0;
for t = 1:T
    Xi = problem.data(t).X;
    yi = problem.data(t).y;
    XitXi = Xi'*Xi;
    Xityi = Xi'*yi;
    w = (options.lambda*eye(problem.r) + (Uopt'*XitXi)*Uopt)\(Uopt'*Xityi);
    nmse_refined = nmse_refined + calculateErr(problem.data_test(t).X*(Uopt*w), problem.data_test(t).y, 'nmse');
end
nmse_refined_online = nmse_refined/T;


%% Call parallel algorithm

% Options not mandatory. If using default options, replace options = [].
options.stepsize = 1e-7; % Initial stepsize.
options.rho = 1e5; % The weight factor.
options.lambda = 1e-1; % The ridge regression parameter for computing W.
options.stepsize_decay = 1e3; % The stepsize decay rate.
options.iters = 400; % Number of iterations.
options.verbosity = 1; % Verbosity.

[U_cells_parallel, infos_parallel, options_parallel] = parallel_decentralized_MTL_feat([], problem, options);


% NMSE assuming that the agents are in consensus.
T = length(problem.data); % Total number of tasks.
nmse = 0;
for i = 1:problem.N
    nmse = nmse + length(infos_parallel{i}.tasks)*infos_parallel{i}.cost_test(end);
end
nmse_parallel = nmse/T;

% NMSE after refinement with Karcher mean computation.
Uopt = karchermean_Grassmann(U_cells_parallel); % This gives an "average" point.
nmse_refined = 0;
for t = 1:T
    Xi = problem.data(t).X;
    yi = problem.data(t).y;
    XitXi = Xi'*Xi;
    Xityi = Xi'*yi;
    w = (options.lambda*eye(problem.r) + (Uopt'*XitXi)*Uopt)\(Uopt'*Xityi);
    nmse_refined = nmse_refined + calculateErr(problem.data_test(t).X*(Uopt*w), problem.data_test(t).y, 'nmse');
end
nmse_refined_parallel = nmse_refined/T;






%% Plots

pp = 10;


% Time
N = problem.N; 
fs = 20;
figure;
% N+1 is a special structure of storing other information.
plot(infos_online{N+1}.iter(1:pp:end), infos_online{N+1}.time(1:pp:end),'-O','Color','r','LineWidth',2, 'MarkerSize',13);
hold on;
% Normalize the iterations.
plot(infos_parallel{N+1}.iter(1:pp:end)*((N-1)/2),infos_parallel{N+1}.time(1:pp:end),'-s','Color','b','LineWidth',2, 'MarkerSize',13);
hold off;
ax1 = gca;
set(ax1,'FontSize',fs);
xlabel(ax1,['Every ',num2str(pp),'th iteration'],'FontSize',fs);
ylabel(ax1,'Time in seconds','FontSize',fs);
legend('Online', 'Parallel');
legend 'boxoff';
box off;


