
clear; clc; close all;

%
T = 1000; % Num tasks.
m = 100; % Num features.
r = 5;

[Q, ~] = qr(randn(m, r),0);
Qopt = Q;
Q = Q*Q';

W = randn(m,T);

% Syenthetic train data
for ii = 1 : T
    j = randi([10, 50]); % Random training examples between 100 and 500.
    data(ii).X = randn(j, m); % X for task ii.
    w = randn(m, 1); %W(:,ii);%
    data(ii).y = data(ii).X*Q*w + 1e-6*(rand(1) - 0.5); % y for task ii.
    %     std(data(ii).y)^2
end


N = 6;
problem.data = data;

% When you have data_test that follows the
% same structure of data
problem.data_test = data;
problem.r = r;
problem.N = N;
problem.m = m;
problem.Uopt = Qopt; % Target subspace.


%% Proposed
clear params;

params.iters = 200*(N-1);
params.stepsize = 1e-4; %1e-4 works good.
params.rho = 1e3;
params.lambda = 0;
params.stepsize_decay = 1e-3*1/params.stepsize;

[U_cells, infos] = online_decentralized_MTL_feat([], problem, params);

% NMSE assuming that the agents are in consensus.
nmse = 0;
for i = 1:problem.N
    nmse = nmse + length(infos{i}.tasks)*infos{i}.cost_test(end);
end
nmse = nmse/T;

% NMSE after refinement with Karcher mean computation.
Uopt = karchermean_Grassmann(U_cells);
nmse1 = 0;
for t = 1:T
    Xi = problem.data(t).X;
    yi = problem.data(t).y;
    XitXi = Xi'*Xi;
    Xityi = Xi'*yi;
    w = (params.lambda*eye(problem.r) + (Uopt'*XitXi)*Uopt)\(Uopt'*Xityi);
    nmse1 = nmse1 + calculateErr(problem.data_test(t).X*(Uopt*w), problem.data_test(t).y, 'nmse');
end
nmse1 = nmse1/T;




%% Alt minimization Algorithm of Argyriou et al (2008)
% % For Parkinson, mu = 1, lambda = 1e3.
% % For School, mu = 1, lambda = 1.
% % For Sarcos, mu = 1, lambda = 1.
% 
% 
% clear options;
% mu = 1e-10;
% lambda = 1e3;
% maxiter = 2*200;
% 
% options.mu = mu;
% options.lambda = lambda;
% options.maxiter= maxiter;
% options.computestats = true; % If we want to compute the test rmsse, make it true.
% 
% [Omega_alt, W_alt, infos_alt] = train_alternating_epsilon_Argyriou(problem,eye(m)/m,options);
% test_error_alt = infos_alt.test;
% nmse_alt = mean(infos_alt.test(end, :));

%% Plots
pp = 10;

% Distance to solution
fs = 20;
figure;
semilogy(infos{1}.disttoUopt(1:pp:end),'-+','Color','m','LineWidth',2, 'MarkerSize',13);
hold on;
semilogy(infos{2}.disttoUopt(1:pp:end),'-*','Color','b','LineWidth',2, 'MarkerSize',13);
semilogy(infos{3}.disttoUopt(1:pp:end),'-d','Color','k','LineWidth',2, 'MarkerSize',13);
semilogy(infos{4}.disttoUopt(1:pp:end),'-O','Color','r','LineWidth',2, 'MarkerSize',13);
semilogy(infos{5}.disttoUopt(1:pp:end),'-s','Color','c','LineWidth',2, 'MarkerSize',13);
semilogy(infos{6}.disttoUopt(1:pp:end),'-v','Color','g','LineWidth',2, 'MarkerSize',13);
hold off;
ax1 = gca;
set(ax1,'FontSize',fs);
xlabel(ax1,['Every ',num2str(pp),'th update'],'FontSize',fs);
ylabel(ax1,'Distance to the optimal subspace','FontSize',fs);
legend('Agent 1', 'Agent 2','Agent 3','Agent 4','Agent 5','Agent 6');
legend 'boxoff';
box off;





